package ca.recrown.islandsurvivalcraft.interaction.items;

import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.entity.HumanEntity;
import org.bukkit.inventory.EquipmentSlot;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.Recipe;

import ca.recrown.islandsurvivalcraft.IslandSurvivalCraft;

public interface VariedItem {
    /**
     * Should be used to perform anything that need only be called once.
     * Is called only when the needInitialization method returns true.
     * Is called after assigning identity.
     * @param islandSurvivalCraft The IslandSurvivalCraft plugin object that contains references to various managers that may be used.
     */
    public void initialize(IslandSurvivalCraft islandSurvivalCraft);

    /**
     * @return the recipe to be registered with bukkit.
     */
    public Recipe getRecipe();

    /**
     * @return The official, displayed name of the item.
     */
    public String getName();

    /**
     * 
     * @return The material item this item is a variation of.
     */
    public Material getBaseItem();

    /**
     * @return The namespaced key Bukkit will use to identify this variation of the item.
     */
    public NamespacedKey getNamespacedKey();

    /**
     * Assigns a persistent metadata identifier.
     * This identifier should be saved to the recipe's resulting itemstack.
     * @param identifier The identifier.
     */
    public void assignIdentifier(VariationItemIdentifier identifier);

    /**
     * Should give the identifier used identify this variation of the base item from the original.
     * This usually should just return the assigned identifier.
     * @return The identifier metadata.
     */
    public VariationItemIdentifier getIdentifier();

    /**
     * If this item needs initializing.
     * If returning true, the initialize method will be called before being registered as a recipe.
     * @return True if the initialize method should be called.
     */
    public boolean needInitialization();

    /**
     * Called when this variety of the item is crafted.
     * @param item the resulting itemstack.
     * @param crafter the entity performing the crafting.
     */
    public void onItemCrafted(ItemStack item, HumanEntity crafter);
    
    /**
     * Called when this item is right clicked with.
     * @param item The itemstack that was right clicked with.
     * @param clicker The player that did the clicking.
     * @param hand The hand that it was in.
     * @param block The block the item was right clicked on.
     * @param blockFace The face of the block that was clicked.
     * @return
     */
    public boolean onItemRightClicked(ItemStack item, HumanEntity clicker, EquipmentSlot hand, Block block, BlockFace blockFace);
}